import 'dart:convert';
import 'package:dio/dio.dart';
import 'package:dio_log/dio_log.dart';
import 'package:flutter/foundation.dart';
import 'package:jm_basic_flutter/jm_basic_flutter_platform_interface.dart';
import 'package:jm_basic_flutter/net/network_config_mixin.dart';
import 'package:jm_basic_flutter/net/network_request_entity.dart';
import 'package:jm_basic_flutter/utils/common_util.dart';
import 'package:jm_basic_flutter/utils/toast_util.dart';
import 'http_error.dart';

typedef HttpSuccessCallback<T> = void Function(T data);
typedef HttpErrorCallback<T> = void Function(HttpError data);
typedef HttpMessageAndCodeCallback<T> = void Function(String message, int code);
typedef HttpCompleteCallback<T> = void Function();

final Dio dio = Dio();
late NetworkConfigMixin dioConfig;

initDio(NetworkConfigMixin config) {
  dioConfig = config;
  BaseOptions options = BaseOptions(
    baseUrl: dioConfig.baseUrl,
    connectTimeout: 30000,
    receiveTimeout: 30000,
    sendTimeout: 30000,
    headers: dioConfig.basicHeaders,
  );
  dio.options = options;
  // dio log
  // ios 调用的是原生请求，wap 不需要在控制台打印信息
  if (!kIsIOS) {
    dio.interceptors.add(DioLogInterceptor());
    DioLogInterceptor.enablePrintLog = kIsAndroid && !kReleaseMode;
  }
  if (kIsAndroid && !kReleaseMode) {
    dio.interceptors.add(LogInterceptor(
      requestHeader: true,
      requestBody: true,
      responseHeader: true,
      responseBody: true,
    )); //开启请求日志
  }
}

class DioClient {
  final CancelToken cancelToken;

  DioClient(this.cancelToken);

  DioClient.single({
    required this.cancelToken,
  });

  void post<T>({
    required NetworkRequestEntity request,
    bool withLoading = true,
    bool withMessage = true,
    bool clickClose = false,
    bool allowClick = true,
    bool ignoreContentClick = true,
    HttpSuccessCallback<T?>? successCallback,
    HttpMessageAndCodeCallback<T>? messageAndCodeCallback,
    HttpCompleteCallback<T>? completeCallback,
    HttpErrorCallback<T>? errorCallback,
  }) async {
    _httpRequest("POST",
        request: request,
        successCallback: successCallback,
        withLoading: withLoading,
        withMessage: withMessage,
        clickClose: clickClose,
        allowClick: allowClick,
        ignoreContentClick: ignoreContentClick,
        messageAndCodeCallback: messageAndCodeCallback,
        completeCallback: completeCallback,
        errorCallback: errorCallback);
  }

  void put<T>({
    required NetworkRequestEntity request,
    bool withLoading = true,
    bool withMessage = true,
    bool clickClose = false,
    bool allowClick = true,
    bool ignoreContentClick = true,
    HttpSuccessCallback<T?>? successCallback,
    HttpMessageAndCodeCallback<T>? messageAndCodeCallback,
    HttpCompleteCallback<T>? completeCallback,
    HttpErrorCallback<T>? errorCallback,
  }) async {
    _httpRequest("PUT",
        request: request,
        successCallback: successCallback,
        withLoading: withLoading,
        withMessage: withMessage,
        clickClose: clickClose,
        allowClick: allowClick,
        ignoreContentClick: ignoreContentClick,
        messageAndCodeCallback: messageAndCodeCallback,
        completeCallback: completeCallback,
        errorCallback: errorCallback);
  }

  void get<T>({
    required NetworkRequestEntity request,
    bool withLoading = true,
    bool withMessage = true,
    bool clickClose = false,
    bool allowClick = true,
    bool ignoreContentClick = true,
    HttpSuccessCallback<T?>? successCallback,
    HttpMessageAndCodeCallback<T>? messageAndCodeCallback,
    HttpCompleteCallback<T>? completeCallback,
    HttpErrorCallback<T>? errorCallback,
  }) async {
    _httpRequest("GET",
        request: request,
        successCallback: successCallback,
        withLoading: withLoading,
        withMessage: withMessage,
        clickClose: clickClose,
        allowClick: allowClick,
        ignoreContentClick: ignoreContentClick,
        messageAndCodeCallback: messageAndCodeCallback,
        completeCallback: completeCallback,
        errorCallback: errorCallback);
  }

  void delete<T>({
    required NetworkRequestEntity request,
    bool withLoading = true,
    bool withMessage = true,
    bool clickClose = false,
    bool allowClick = true,
    bool ignoreContentClick = true,
    HttpSuccessCallback<T?>? successCallback,
    HttpMessageAndCodeCallback<T>? messageAndCodeCallback,
    HttpCompleteCallback<T>? completeCallback,
    HttpErrorCallback<T>? errorCallback,
  }) async {
    _httpRequest("DELETE",
        request: request,
        successCallback: successCallback,
        withLoading: withLoading,
        withMessage: withMessage,
        clickClose: clickClose,
        allowClick: allowClick,
        ignoreContentClick: ignoreContentClick,
        messageAndCodeCallback: messageAndCodeCallback,
        completeCallback: completeCallback,
        errorCallback: errorCallback);
  }

  void _httpRequest<T>(
    String method, {
    required NetworkRequestEntity request,
    required bool withLoading, // 是否加载 loading
    required bool withMessage, // 是否弹窗展 message
    required bool clickClose,
    required bool allowClick,
    required bool ignoreContentClick, // loading 背景是否可以穿透
    HttpSuccessCallback<T?>? successCallback,
    HttpMessageAndCodeCallback<T>? messageAndCodeCallback,
    HttpCompleteCallback<T>? completeCallback,
    HttpErrorCallback<T>? errorCallback,
  }) async {
    if (withLoading) {
      ToastUtil.showTransparentLoading(
        clickClose: clickClose,
        allowClick: allowClick,
        ignoreContentClick: ignoreContentClick,
      );
    }
    try {
      // //检查网络是否连接
      // ConnectivityResult connectivityResult = await (Connectivity().checkConnectivity());
      // if (connectivityResult == ConnectivityResult.none) {
      //   throw (HttpError(HttpError.NETWORK_ERROR, "网络异常，请稍后重试！"));
      // }
      dio.options.method = method;
      dynamic getParameters;
      dynamic postData;
      if (method == "POST" || method == "PUT") {
        postData = request.parameters();
      }
      if (method == "GET" || method == "DELETE") {
        getParameters = request.parameters();
      }

      final urlString = request.url() ?? '';
      final headers = request.requestHeaders();

      dio.options.headers = headers;
      int code;
      String message = '';
      dynamic responseDecodedData;

      final isFileUpload = headers['Content-Type'] == 'multipart/form-data';
      if (kIsIOS && !isFileUpload) {
        final nativeParameters = request.parameters();
        dioConfig.iosRequestPrint(method, urlString, headers, parameters: nativeParameters);
        Map<String, dynamic> nativeData = {
          'method': method,
          'urlString': urlString,
          'headers': headers,
        };
        if (nativeParameters != null) {
          nativeData['parameters'] = nativeParameters;
        }
        if ((request.debugJson ?? '').isEmpty) {
          final iosResponse = await JmBasicFlutterPlatform.instance.callIOSNativeRequest(nativeData);
          final isSuccess = iosResponse?['isSuccess'] ?? false;
          final statusCode = iosResponse?['statusCode'] ?? -1;
          final iosResponseData = iosResponse?['data'];
          if (!isSuccess) {
            throw Exception([statusCode, message]);
          } else if (iosResponseData == null) {
            throw Exception([statusCode, 'Request success but response data is null']);
          } else {
            final iosResponseDecodedData = jsonDecode(utf8.decode(iosResponseData)) as Map;
            dioConfig.iosResponsePrint(method, urlString, headers, iosResponseDecodedData, parameters: nativeParameters);
            if (iosResponseDecodedData["code"] is String) {
              code = int.parse(iosResponseDecodedData["code"]);
            } else {
              code = iosResponseDecodedData["code"];
            }
            message = iosResponseDecodedData['message'] ?? '';
            responseDecodedData = iosResponseDecodedData['data'];
          }
        } else {
          final parseData = dioConfig.debugJsonParseForIos(headers, request.debugJson ?? '');
          code = parseData['statusCode'];
          responseDecodedData = parseData['data'];
        }
      } else {
        if (isFileUpload) {
          dioConfig.iosRequestPrint(method, urlString, headers, parameters: getParameters);
        }
        Response responseBody;
        if ((request.debugJson ?? '').isEmpty) {
          responseBody = await dio.request(urlString, queryParameters: getParameters, data: postData, cancelToken: cancelToken);
        } else {
          responseBody = dioConfig.debugJsonParseForNormal(headers, urlString, request.debugJson ?? '');
        }
        if (responseBody.data["code"] is String) {
          code = int.parse(responseBody.data["code"]);
        } else {
          code = responseBody.data["code"];
        }
        message = responseBody.data['message'] ?? '';
        responseDecodedData = responseBody.data["data"];
        if (isFileUpload) {
          dioConfig.iosResponsePrint(method, urlString, headers, responseBody.data, parameters: getParameters);
        }
      }

      final isStringList = (kIsWeb && <String>[] is T) || (!kIsWeb && <String>[].runtimeType == T);
      final isIntList = (kIsWeb && <int>[] is T) || (!kIsWeb && <int>[].runtimeType == T);
      final isDoubleList = (kIsWeb && <double>[] is T) || (!kIsWeb && <double>[].runtimeType == T);

      if (code == 200) {
        if (successCallback != null) {
          if (responseDecodedData == null) {
            successCallback(null);
          } else if (responseDecodedData is String || responseDecodedData is int || responseDecodedData is double || responseDecodedData is bool) {
            successCallback(responseDecodedData);
          } else if (isStringList) {
            successCallback(responseDecodedData.map<String>((e) => e.toString()).toList() as T);
          } else if (isIntList) {
            successCallback(responseDecodedData.map<int>((e) => (e is int ? e : int.tryParse(e))).toList() as T);
          } else if (isDoubleList) {
            successCallback(responseDecodedData.map<double>((e) => (e is double ? e : double.tryParse(e))).toList() as T);
          } else {
            successCallback(dioConfig.fromJsonAsT(responseDecodedData));
          }
        }
      } else {
        if (errorCallback != null) {
          errorCallback(HttpError(code.toString(), message));
        }
        // 弹窗处理
        if (withMessage && code != HttpError.UNAUTHORIZED) {
          if (code == HttpError.NOT_FOUND || code == HttpError.INTERNAL_SERVER_ERROR) {
            ToastUtil.show(dioConfig.netRequestFailedTips);
          } else {
            ToastUtil.show(message);
          }
        }
      }
      dioConfig.netRequestSuccess(code, message, urlString);
      if (messageAndCodeCallback != null) {
        messageAndCodeCallback(message, code);
      }
    } on DioError catch (e, s) {
      if (!kIsWeb) {
        debugPrint("请求出错：$e\n$s");
      }
      if (errorCallback != null && e.type != DioErrorType.cancel) {
        errorCallback(HttpError.dioError(e));
      }
      if (withMessage) {
        if (kDebugMode) {
          ToastUtil.show(e.message);
        } else {
          ToastUtil.show(dioConfig.netRequestFailedTips);
        }
      }
      throw (HttpError.dioError(e));
    } catch (e, s) {
      if (!kIsWeb) {
        debugPrint("未知异常出错：$e\n$s");
      }
      if (errorCallback != null) {
        errorCallback(HttpError(HttpError.UNKNOWN, "网络异常，请稍后重试！"));
      }
      throw (HttpError(HttpError.UNKNOWN, "网络异常，请稍后重试！-- $e\n$s"));
    } finally {
      if (withLoading) {
        ToastUtil.dismiss();
      }
      if (completeCallback != null) {
        completeCallback();
      }
    }
  }
}

class CommonInterceptor extends InterceptorsWrapper {
  @override
  void onRequest(RequestOptions options, RequestInterceptorHandler handler) {
    return handler.next(options);
  }

  @override
  void onResponse(Response response, ResponseInterceptorHandler handler) {
    return handler.next(response);
  }

  @override
  void onError(DioError err, ErrorInterceptorHandler handler) {
    return handler.next(err);
  }
}
